/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// constants
const ffIError = Components.interfaces.ffIError;
const ffIDisk = Components.interfaces.ffIDisk;

const FF_TIME_NIGHT = -1; // time that night switches to day
const FF_TIME_DAY = 15;  // time that day switches to night

//global forecastfox object
var gForecastfox = null;

function ffLoad()
{
  gForecastfox = new Forecastfox();
  gForecastfox.start();
};

function ffUnload()
{
  gForecastfox.stop();
  gForecastfox = null;
};

///////////////////////
// Forecastfox Object
function Forecastfox() {};
Forecastfox.prototype = {
  _manager: null,
  _bundle: null,
  _prefixes: null,
  
  start: function Forecastfox_start()
  {
    //get our XPCOM manager service
    var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
    this._bundle = document.getElementById("ff-bundle");  
    
    //setup object
    this._prefixes = new Object();
    this._prefixes["cc"] = "current";
    this._prefixes["dayf"] = "forecast";
    this._prefixes["dayt"] = "forecast";

    //add observer to forecastfox:update
    obs.addObserver(this, "forecastfox:manager", false);
    
    //add resize listener
    //XXX comment out until crasher bug 119 is fixed
    //var comp = this;
    //window.addEventListener("resize", function resize(event) { comp.updateOverflow(event); }, false);

    //setup display - give window a chance to display before loading
    if (this._manager.firstTime)
      setTimeout("gForecastfox._manager.load(false);", 100);
    else
      setTimeout("gForecastfox._update(null, '');", 10);
  },

  stop: function Forecastfox_stop()
  {
    try {
      //remove observers
      var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obs.removeObserver(this, "forecastfox:manager");
      obs = null;
      
      //XXX comment out until crasher bug 119 is fixed
      //var comp = this;
      //window.removeEventListener("resize", function resize(event) { comp.updateOverflow(event); }, false);
      //comp = null;      
    } catch(e) {};

    //destroy variables
    this._manager = null;
    this._bundle = null;
    this._prefixes = null;
  },
  
  observe: function Forecastfox_observe(aSubject, aTopic, aData)
  {
    var method = null;
    var data = null;
    if (aData.indexOf(",") != -1) {
      method = aData.substring(0, aData.indexOf(","));
      data = aData.substring(aData.indexOf(",") + 1, aData.length);
    } else
      method = aData;
    switch (method) {
      case "move":
      case "profile":
      case "icons":
        setTimeout("gForecastfox._update(null, 'end');", 10);
        break;
      case "data":
        var group = this._prefixes[data];
        this._update(group, "end");
        break;
      default:
        this._update(true, method);
        break;
    }
  },

  hideTooltip: function Forecastfox_hideTooltip(aNode)
  {
    if (!aNode.hasAttribute("tooltip"))
      return;
      
    var tooltip = document.getElementById(aNode.getAttribute("tooltip"));
    tooltip.hidePopup();
  },
  
  hideSubmenus: function Forecastfox_hideSubmenus()
  {
    var menus = new Array();
    menus.push("ff-profs-popup");
    menus.push("ff-icons-popup"); 
    menus.push("ff-popup");       
    
    for (var x=0; x<menus.length; x++) {
      var menu = document.getElementById(menus[x]);    
      menu.hidePopup();
    };
  },
  
  selectProfile: function Forecastfox_selectProfile(aEvent)
  { 
    //set if not current
    var item = aEvent.target;
    var profile = item.getAttribute("label");
    if (profile != this._manager.getUTFPref("profile.current"))
      this._manager.setUTFPref("profile.current", profile);
  },
  
  selectIcons: function Forecastfox_selectIcons(aEvent)
  {    
    //set if not current
    var item = aEvent.target;
    var icon = item.getAttribute("id");
    if (icon != this._manager.icons.current.id && icon != "ff-icons-more")
      this._manager.setUTFPref("icons.current", icon);
  },
  
  addSubmenus: function Forecastfox_addSubmenus(aEvent)
  {
    if (aEvent.target.getAttribute("id") != "ff-popup")
      return;
      
    this._addProfiles();
    this._addIcons();    
  },
  
  removeSubmenus: function Forecastfox_removeSubmenus(aEvent)
  {
    if (aEvent.target.getAttribute("id") != "ff-popup")
      return;
        
    var menus = new Array();
    menus.push("ff-profs-popup");
    menus.push("ff-icons-popup"); 
    for (var x=0; x<menus.length; x++) {
      var menu = document.getElementById(menus[x]);
      while (menu.hasChildNodes() && menu.lastChild.tagName != "menuseparator")
        menu.removeChild(menu.lastChild);
    };      
  },
  
  clickLink: function Forecastfox_clickLink(aEl, event)
  {
    //middle clicked  
    if (event.button == 1) {
    
      //do oncommand
      eval(aEl.getAttribute("oncommand"));    
    };
  },
   
  openLink: function Forecastfox_openLink(aEvent, aUnits, aPartner, aFrom)
  {
    // supply an aOpenEvent and it will be used in the openUILink instead of the aEvent
    var url = aEvent.target.getAttribute("url");
    if (url == "") {
      url = this._bundle.getString("ff.url.default");
      aPartner = true;
      aUnits = false;
    };
    
    var separator = (url.indexOf("?") != -1) ? "&" : "?";
    if (aPartner)
      url += separator + "partner=" + this._manager.partner;
    if (aUnits) {
      separator = (url.indexOf("?") != -1) ? "&" : "?";
 	    url += separator + "metric=" + this._manager.branch.getComplexValue("units.current",Components.interfaces.nsIPrefLocalizedString).data;
    };

    this._manager.openLink(url, this._getWhere(aEvent, aFrom));
  },
  
  updateOverflow: function Forecastfox_updateOverflow(aEvent)
  {
    // Ignore events that aren't on the document or the window
    if (aEvent && aEvent.target != document && aEvent.target != window)
      return;
        
    var comp = this;
    if (aEvent && aEvent.type == "focus")     
      window.removeEventListener("focus", function resize(event) { comp.updateOverflow(event); }, false);
          
    //get elements  
    var chevron = document.getElementById("ff-overflow-btn");
    var deck = document.getElementById("ff-deck");
    var box = document.getElementById("ff-box");
    var toolbar = box.parentNode;
    var popup = document.getElementById("ff-overflow-popup");
    
    if (toolbar.boxObject.width <= 0) {
      window.addEventListener("focus", function resize(event) { comp.updateOverflow(event); }, false);
      return;
    };
    
    //reset the display                  
    chevron.setAttribute("collapsed", "true");
    if (deck.parentNode != box) {
      deck.parentNode.removeChild(deck);
      box.appendChild(deck);
    };
    var orient = document.defaultView.getComputedStyle(toolbar, "").getPropertyValue("-moz-box-orient")
    this._setOrient(orient);
    
    //determine space needed for other elements
    var leftSize = 0;
    var neededSize = this._getElementSize(deck, orient);
    var items = document.getAnonymousNodes(toolbar, orient);
    if (!items)
      items = toolbar.childNodes; 
    for (var x=0; x<items.length; x++) {
      var item = items[x];
      if (item == box)
        continue;
          
      leftSize += this._getElementSize(item, orient);
    };
        
    //overflow needed
    var totalSize = (orient == "horizontal") ? window.innerWidth : window.innerHeight;
    if (neededSize > (totalSize - leftSize)) {
      this._setOrient("vertical");
      deck.parentNode.removeChild(deck);
      popup.appendChild(deck);
      chevron.removeAttribute("collapsed");
    }; 
  },
  
  _getElementSize: function Forecastfox__getElementSize(aElement, aOrient)
  {
    //setup variables
    var style = document.defaultView.getComputedStyle(aElement, "");
    var margin1 = 0;
    var margin2 = 0;
    var size = 0;
    
    //get width
    if (aOrient == "horizontal") {
      margin1 = style.getPropertyValue("margin-left");
      margin1 = margin1 ? Math.round(parseFloat(margin1)) : 0;
      margin2 = style.getPropertyValue("margin-right");
      margin2 = margin2 ? Math.round(parseFloat(margin2)) : 0;
      size = aElement.boxObject.width;
    
    //get heigh  
    } else {
      margin1 = style.getPropertyValue("margin-top");
      margin1 = margin1 ? Math.round(parseFloat(margin1)) : 0;
      margin2 = style.getPropertyValue("margin-bottom");
      margin2 = margin2 ? Math.round(parseFloat(margin2)) : 0;
      size = aElement.boxObject.height;
    };
    
    return size + margin1 + margin2;  
  },
  
  _setOrient: function Forecastfox__setOrient(aOrient)
  {
    //element that need set
    var ids = ["ff-box", "ff-weather", "ff-current-box", "ff-spacer", "ff-error",
      "ff-forecast-box", "ff-today-panel", "ff-forecast-panel", "ff-progress"];
      
    //set attribute
    for (var x=0; x<ids.length; x++)
      document.getElementById(ids[x]).setAttribute("orient", aOrient);
  },
  
  _addIcons: function Forecastfox__addIcons()
  {
    var menu = document.getElementById("ff-icons-popup");
    var keys = this._manager.icons.getItemList({});
        
    //set none
    var item;    
    if (keys.length == 0) {
      item = document.createElement("menuitem");
      item.setAttribute("label", this._bundle.getString("ff.icons.none"));
      item.setAttribute("disabled", true);
      menu.appendChild(item);
      return;
    };
    
    //set for one or more
    for (var x=0; x<keys.length; x++) {
      item = document.createElement("menuitem");
      item.setAttribute("label", keys[x].name);
      item.setAttribute("id", keys[x].id);
      item.setAttribute("type", "radio");
      item.setAttribute("name", "icon");     
      if (keys[x].id == this._manager.icons.current.id)
        item.setAttribute("checked", "true");
      menu.appendChild(item);
    };     
  },
    
  _addProfiles: function Forecastfox__addProfiles()
  {
    var menu = document.getElementById("ff-profs-popup");
    var count = this._manager.profiles.getProfileNum();
    var keys = new Array();
    for (var x = 0; x < count; x++)
    	keys[x] = this._manager.profiles.getProfile(x);
    keys.sort();
        
    //set none
    var item;    
    if (keys.length == 0) {
      item = document.createElement("menuitem");
      item.setAttribute("label", this._bundle.getString("ff.profiles.none"));
      item.setAttribute("disabled", true);
      menu.appendChild(item);
      return;
    };  
    
    //set for one or more
    for (var x=0; x<keys.length; x++) {
      item = document.createElement("menuitem");
      item.setAttribute("label", keys[x]);
      item.setAttribute("type", "radio");
      item.setAttribute("name", "profile");     
      if (keys[x] == this._manager.getUTFPref("profile.current"))
        item.setAttribute("checked", "true");
      menu.appendChild(item);
    };     
  },
    
  _getWhere: function Forecastfox__getWhere(aEvent, aFrom)
  {
    //get left click default where
    var where = this._manager.branch.getCharPref("links." + aFrom);
    
    //get link modifier
    try {
      var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var appBranch = pbs.getBranch(null);
      var background = appBranch.getBoolPref("browser.tabs.loadInBackground");
    } catch(e) {
      background = false;
    };
    
    //determine if middle click 
    var middle = (aEvent.button == 1);
    
    //get keys pressed
    var ctrl = aEvent.ctrlKey;
    var shift = aEvent.shiftKey;
    
    //do key based modifiers
    if (ctrl || middle) {
      if (shift || background)
        where = "tabshifted";
      else
        where = "tab";
    } else if (shift)
      where = "window";
          
    return where;
  },
   
  _moveBox: function Forecastfox__moveBox()
  {
    //check if box is in the right location
    var toolbar = document.getElementById(this._manager.branch.getCharPref("general.bar"));
    var box = document.getElementById("ff-box");
    var position = this._manager.branch.getIntPref("general.position");
    
    //if toolbar doesn't exist move to default location
    if (!toolbar) {
      this._manager.branch.setCharPref("general.bar", "status-bar");
      this._manager.branch.setIntPref("general.position", -1);
      return;
    };
    
    //XXX remove orient code when bug 119 is fixed.  It exists in updateOverflow function
    var orient = document.defaultView.getComputedStyle(toolbar, "").getPropertyValue("-moz-box-orient")
    this._setOrient(orient);
        
    // bail if it is in the right place
    if (this._indexOf(toolbar,box) == position)
      return;

    //remove us from parent
    box.parentNode.removeChild(box);
    
    //make sure we have the right element type
    var newbox = null;
    if (toolbar.localName == "statusbar") {
      if (box.localName != "statusbarpanel")
        newbox = document.createElement("statusbarpanel");
    } else {
      if (box.localName == "statusbarpanel")
        newbox = document.createElement("hbox");    
    };
    
    //append children of old box
    if (newbox) {
      newbox.setAttribute("id", "ff-box");
      while (box.hasChildNodes())
        newbox.appendChild(box.firstChild);      
    } else
      newbox = box;
    
    //insert us in correct place
    this._insertAtIndex(toolbar, newbox, position);
  },
  
  _indexOf: function Forecastfox__indexOf(aParent, aChild)
  {
    // -1 if it does not exist
    var children = aParent.childNodes;
    for (var x=0; x<children.length; x++)
      if (children[x] == aChild)
        return (x == children.length - 1) ? -1 : x;
   
    return null;
  },
  
  _insertAtIndex: function Forecastfox__insertAtIndex(aParent, aChild, aIndex)
  {
    var children = aParent.childNodes;
    if ((children.length == 0) || (aIndex >= children.length) || (aIndex < 0))
      aParent.appendChild(aChild);
    else
      aParent.insertBefore(aChild, children[aIndex]);
  },
  
  _update: function Forecastfox__update(aGroup, aMethod)
  {
    var error;
    
    //update all
    if (!aGroup) {
      try {
        error = this._manager.getError("current");
        if (error.severity != ffIError.SEVERITY_INFO)
          this._update("current", "error");
        else
          this._update("current", "end");
      } catch (e) {
        this._manager.disk.recordError("Message: It looks like manager.getError may have returned a null.", e);
        this._update("current", "end");
      };
      return;
    };
    
    //move box
    this._moveBox();
    
    //remove progress and errors
    this._removeError();
    this._removeProgress();
    
    //now do update
    switch (aMethod) {
      case "start":
        this._showProgress();
        break;
      case "error":
        this._showError();
        break;
      case "end":
      default:
        this._showCurrent();
        this._showForecast();
        break;
    };
    
    this._borders();
    
    //setup overflow
    //XXX comment out until crasher bug 119 is fixed
    //this.updateOverflow();
  },

  _removeError: function Forecastfox__removeError()
  {
    var item = document.getElementById("ff-error"); 
    
    //remove error  
    this._removeTooltip(item);
    item.removeAttribute("value");
    item.setAttribute("collapsed", true);
  },
  
  _removeProgress: function Forecastfox__removeProgress()
  {
    var item = document.getElementById("ff-progress");
    
    //remove progress
    this._removeTooltip(item);
    item.setAttribute("mode", "determined");
    item.setAttribute("value", "0");
    item.setAttribute("collapsed", "true");
  },
  
  _removeTooltip: function Forecastfox__removeTooltip(aItem)
  {
    //remove tooltiptext
    if (aItem.hasAttribute("tooltiptext")) {
      aItem.removeAttribute("tooltiptext");
    };
    
    //remove tooltip elements
    if (aItem.hasAttribute("tooltip")) {
      var tooltip = document.getElementById(aItem.getAttribute("tooltip"));
      tooltip.parentNode.removeChild(tooltip);
      aItem.removeAttribute("tooltip");   
    } 
  },

  _showError: function Forecastfox__showError()
  {
    var item = document.getElementById("ff-error");   
    
    //set error
    try {
      var error = this._manager.getError("current");
      item.setAttribute("value", error.description);
      item.setAttribute("tooltiptext", error.tooltip);
    } catch (e) {
      this._manager.disk.recordError("Message: It looks like manager.getError may have returned a null.", e);
      item.setAttribute("value", "");
      item.setAttribute("tooltiptext", "");
    };
    
    //show panel
    item.removeAttribute("collapsed");
    item.parentNode.setAttribute("selectedIndex", "1");      
  },
  
  _showProgress: function Forecastfox__showProgress()
  {
    var item = document.getElementById("ff-progress");   
    
    //set progress
    item.removeAttribute("value");
    item.setAttribute("mode", "undetermined");
    item.setAttribute("tooltiptext", this._bundle.getString("ff.tooltip.loading"));    
    
    //show panel
    item.removeAttribute("collapsed");
    item.parentNode.setAttribute("selectedIndex", "2");  
  },

  _showCurrent: function Forecastfox__showCurrent()
  {
    var current = this._getItem("current", "panel");
    var radar = this._getItem("radar", "panel");
    var swa = this._getItem("swa", "panel");
       
    //remove tooltip and attributes
    this._removeTooltip(current);
    current.removeAttribute("src");
    current.removeAttribute("label");
    this._removeTooltip(radar);
    radar.removeAttribute("src");
    radar.removeAttribute("label");
    this._removeTooltip(swa);
    swa.removeAttribute("src");
    swa.removeAttribute("label");
        
    //do current
    if (!this._manager.branch.getBoolPref("cc.panel.enabled"))
      current.setAttribute("collapsed", "true");
    else {
      this._setPanel(current, "cc", null, null);
      current.setAttribute("url", this._manager.parser.getValue("current", "url", null));
      current.removeAttribute("collapsed"); 
    };
    
    //do radar
    if (!this._manager.branch.getBoolPref("radar.panel.enabled"))    
      radar.setAttribute("collapsed", "true");
    else {
      this._setPanel(radar, "radar", "radar", null);
      
      radar.setAttribute("url", this._manager.parser.getValue("radar", "url", null));
      
      // only display the radar node if there is radar
      if (this._manager.branch.getCharPref("radar.cache") != "")
        radar.removeAttribute("collapsed");
      else
        radar.setAttribute("collapsed", true);
    };
    
    //do swa
    if (!this._manager.branch.getBoolPref("swa.panel.enabled") || this._manager.parser.getValue("swa", "active", null) != "true")    
      swa.setAttribute("collapsed", "true");
    else {
      this._setPanel(swa, "swa", "swa", null);
      
      swa.setAttribute("url", this._manager.parser.getValue("swa", "url", null));
      swa.removeAttribute("collapsed");       
    };
    
    //show panel
    var deck = document.getElementById("ff-deck");
    deck.setAttribute("selectedIndex", "0");        
  },
  
  _showForecast: function Forecastfox__showForecast()
  {
    var today = this._getItem("today", "panel");
    var extended = this._getItem("forecast", "panel");
       
    //remove tooltip and attributes
    while (today.hasChildNodes()) {
      this._removeTooltip(today.lastChild);
      today.lastChild.removeAttribute("src");
      today.lastChild.removeAttribute("label");
      today.removeChild(today.lastChild);
    };
    while (extended.hasChildNodes()) {
      this._removeTooltip(extended.lastChild);
      extended.lastChild.removeAttribute("src");
      extended.lastChild.removeAttribute("label");
      extended.removeChild(extended.lastChild);
    };        
    
    //set url for popup menu since it is the same for all forecast days
    var menu = document.getElementById("ff-popup-forecast");
    menu.setAttribute("url", this._manager.parser.getValue("days", "url", null));
    
    //do today
    if (!this._manager.branch.getBoolPref("dayt.panel.enabled") || (this._manager.branch.getIntPref("dayt.panel.mode") < 0))
      today.setAttribute("collapsed", "true");
    else
      this._updateToday(today);
  
    //do extended
    if (!this._manager.branch.getBoolPref("dayf.panel.enabled") || (this._manager.branch.getIntPref("dayf.panel.days") == 0)) 
      extended.setAttribute("collapsed", "true");
    else
      this._updateExtended(extended);
    
    //show panel
    var deck = document.getElementById("ff-deck");
    deck.setAttribute("selectedIndex", "0");        
  },
  
  _updateToday: function Forecastfox__updateToday(aNode)
  {	
    var mode = this._manager.branch.getIntPref("dayt.panel.mode");
    var sw_mode = this._manager.branch.getIntPref("dayt.panel.switch");
    var panels = (mode == 2) ? 2 : 1;
    var cHours = this._getHour();
    var day_index = 0;
  	
    while (this._shouldIncDay(day_index, cHours))
      day_index++;
  	
    for (var x = 0; x < panels; x++) {
  	
      if (cHours <= FF_TIME_NIGHT || cHours >= FF_TIME_DAY) {
        if (mode == 0 && sw_mode == 1)
          day_index++;
        else if (mode == 2)
          panels--;
      };

      if (this._shouldCreatePanel("today", "days", x, cHours))
        this._createPanel("today", aNode, "days", day_index);
      else if (this._shouldCreatePanel("today", "nights", x, cHours))
        this._createPanel("today", aNode, "nights", day_index);
    }
  	
    // show the box!
    aNode.removeAttribute("collapsed");
  },
  
  _updateExtended: function Forecastfox__updateExtended(aNode)
  {
	  var days = this._manager.branch.getIntPref("dayf.panel.days");
    var new_panel = false;
    var cHours    = this._getHour();
    var day_index = 1;
    var day_add   = 0;

    while (this._shouldIncDay(day_add, cHours))
      day_add++;
      
    day_index += day_add;
        
    for (var x = 0; x < days; day_index++) {
      // finish if we went through too many days
      if (day_index > 8)
        break;
        
      try {
        // create the corresponding nodes
        if (this._shouldCreatePanel("extended", "days", day_index, cHours))
          new_panel = this._createPanel("extended", aNode, "days", day_index);
      } catch (e) {}
      
      try {
        if (this._shouldCreatePanel("extended", "nights", day_index, cHours))
          new_panel = this._createPanel("extended", aNode, "nights", day_index);
      } catch (e) {}
      
      // increase x if we made a new one
      if (new_panel)
        x++;
      
      new_panel = false;
    }
    
    aNode.removeAttribute("collapsed");
  },
  
  _shouldIncDay: function Forecastfox__shouldIncDay(aIndex, aHours)
  {    
    return ((aHours < 12 && !this._hasData(aIndex)) && 
    		(FF_TIME_NIGHT < aHours && aHours < FF_TIME_DAY));
  },
  
  _hasData: function Forecastfox__hasData(aIndex)
  {
    return ((this._manager.parser.getValue("days", "t", aIndex) != "N/A") ||
    	    (this._manager.parser.getValue("days", "tlong", aIndex) != "N/A"));
  },
  
  _shouldCreatePanel: function Forecastfox__shouldCreatePanel(aType, aPart, aIndex, aHours) 	 
  {
    var mode, sw_mode, cHours, t_mode, t_sw_mode, t_on;    
    switch (aType) {
      case "today":
        mode = this._manager.branch.getIntPref("dayt.panel.mode");
    	  sw_mode = this._manager.branch.getIntPref("dayt.panel.switch");
    	
        switch (aPart) {
          case "days":
            if (aHours > FF_TIME_NIGHT && aHours < FF_TIME_DAY)
              return ((mode == 0) || (mode == 2 && aIndex == 0));
            else
              return ((mode == 0 && sw_mode == 1) || (mode == 2 && aIndex == 2));
          case "nights":
            if (mode == 1)
              return true;
            else if (aHours > FF_TIME_NIGHT && aHours < FF_TIME_DAY)
              return (mode == 2 && aIndex == 1);
            else
              return ((mode == 0 && sw_mode == 0) || (mode == 2));
        };
        break;
      case "extended":
        mode = this._manager.branch.getIntPref("dayf.panel.mode");
        t_mode = this._manager.branch.getIntPref("dayt.panel.mode");
        t_sw_mode = this._manager.branch.getIntPref("dayt.panel.switch");
        t_on = this._manager.branch.getBoolPref("dayt.panel.enabled");

        switch (aPart) {
          
          case "nights":
            return ((mode == 1) || (mode == 2));
          case "days":
            // if we're displaying days or days and nights
            if (mode == 0 || mode == 2) {
              // If its between the "days" hours
              if (FF_TIME_NIGHT < aHours && aHours < FF_TIME_DAY)
                return true;
              // If its between the "nights" hours
              else
                return ((t_mode != 0) || (!t_on || (aIndex != 1 && t_sw_mode == 1)) || (t_sw_mode != 1));
            }
            else
              return false;
        };
        break;
    };
    
    return false;
  },
  
  _createPanel: function Forecastfox__createPanel(aGroup, aParent, aPart, aDay)
  {
    var node = document.createElement("ffpanel");
    var prefix = (aGroup == "extended") ? "dayf" : "dayt";
    
    //set context menu on node
    node.setAttribute("context","ff-popup");
    node.setAttribute("id", "forecast" + aPart + aDay);
    node.setAttribute("onmouseout", "gForecastfox.hideTooltip(this);");
    node.setAttribute("url", this._manager.parser.getValue(aPart, "url", aDay));

    //set panel and append
    this._setPanel(node, prefix, aPart, aDay);
    aParent.appendChild(node);
    
    // return that one was made
    return true;
  },
      
  _getItem: function Forecastfox__getItem(aItem, aType)
  {
    var item = null;
    item = document.getElementById("ff-" + aItem + "-" + aType);
    return item;  
  },
  
  _getHour: function Forecastfox__getHour()
  {
    var time = this._manager.parser.getValue("local", "tm", null);
    var ind = time.indexOf(":")
    if (ind < 0) {
      time = Number(this._manager.branch.getCharPref("general.last"));
      return (new Date(time)).getHours();
    }
    return Number(time.substring(0,ind));
  },
  
  _setPanel: function Forecastfox__setPanel(aNode, aPrefix, aGroup, aIndex)
  {
    //if group isn't provided get from prefix
    if (!aGroup)
      aGroup = this._prefixes[aPrefix];
      
    //set tooltip
    if (this._manager.branch.getBoolPref(aPrefix + ".tooltip.enabled"))
      this._setTooltip(aNode, aPrefix, aGroup, aIndex);
       
    //set display  
    var icon = this._manager.parser.getValue(aGroup, "icon", aIndex);
    var label = this._manager.parser.setLabel(this._manager.branch.getComplexValue(aPrefix + ".panel.label",Components.interfaces.nsIPrefLocalizedString).data, aGroup, aIndex);
    icon = this._manager.icons.current.getIcon(icon, false);
    
    switch (this._manager.branch.getIntPref(aPrefix + ".panel.display")) {
      case 0:
      default:
        aNode.setAttribute("src", icon.uri);
        aNode.setAttribute("imagewidth", icon.width);
        aNode.setAttribute("imageheight", icon.height);
        break;
      case 1:
        aNode.setAttribute("label", label);
        aNode.setAttribute("crop", "right");
        break;
      case 2:
        aNode.setAttribute("src", icon.uri);
        aNode.setAttribute("imagewidth", icon.width);
        aNode.setAttribute("imageheight", icon.height);
        aNode.setAttribute("label", label); 
        aNode.setAttribute("crop", "right");       
        break;
    };
  },
  
  _setTooltip: function Forecastfox__setTooltip(aNode, aPrefix, aGroup, aIndex)
  {
    //if group isn't provided get from prefix
    if (!aGroup)
      aGroup = this._prefixes[aPrefix];
    
    //create empty tip
    var parent = document.getElementById("ff-popupset");
    var tooltip = document.createElement("tooltip");
    tooltip.setAttribute("id", aNode.getAttribute("id") + "_tip");
    tooltip.setAttribute("noautohide", "true");
    tooltip.setAttribute("type", "fflayout");
    parent.appendChild(tooltip);
            
    //get icon url
    var icon = this._manager.parser.getValue(aGroup, "icon", aIndex);
    icon = this._manager.icons.current.getIcon(icon, true);
    var iconUrl = icon.uri;
    if (aPrefix == "radar")
      iconUrl = this._getFileURL(this._manager.branch.getCharPref("radar.cache"));
        
    //set display 
    var label = this._manager.branch.getComplexValue(aPrefix + ".tooltip.label",Components.interfaces.nsIPrefLocalizedString).data;    
    label = this._manager.parser.setLabel(label, aGroup, aIndex);
    var title = this._manager.branch.getComplexValue(aPrefix + ".tooltip.title",Components.interfaces.nsIPrefLocalizedString).data; 
    title = this._manager.parser.setLabel(title, aGroup, aIndex);      
    switch (this._manager.branch.getIntPref(aPrefix + ".tooltip.display")) {
      case 0:
        tooltip.setAttribute("src", iconUrl);
        break;
      case 1:
        tooltip.setAttribute("title", title);
        tooltip.setAttribute("label", label);
        tooltip.displayLabel();
        break;
      case 2:
        tooltip.setAttribute("src", iconUrl);
        tooltip.setAttribute("title", title);        
        tooltip.setAttribute("label", label);
        tooltip.displayLabel();        
        break;
    };
    
    //set image size
    if (aPrefix != "radar") {
      tooltip.setAttribute("imagewidth", icon.width);
      tooltip.setAttribute("imageheight", icon.height);
    };

    //associate tooltip with panel    
    aNode.setAttribute("tooltip", tooltip.getAttribute("id"));
  },
  
  _borders: function Forecastfox__borders()
  {
    //get elements
    var spacer = document.getElementById("ff-spacer");
    var today = this._getItem("today", "panel");
    var extended = this._getItem("forecast", "panel");
    var radar = this._getItem("radar", "panel");
    var swa = this._getItem("swa", "panel");
    var current = this._getItem("current", "panel");
    var deck = document.getElementById("ff-deck");
    
    var showing = new Object();
    
    //determine what is showing
    showing["deck"] = (deck.getAttribute("selectedIndex") != "0") ? true : false;
    showing["current"] = (this._manager.branch.getBoolPref("cc.panel.enabled") && !current.hasAttribute("collapsed"));
    showing["radar"] = (this._manager.branch.getBoolPref("radar.panel.enabled") && !radar.hasAttribute("collapsed"));
    showing["swa"] = (this._manager.branch.getBoolPref("swa.panel.enabled") && !swa.hasAttribute("collapsed"));
    showing["today"] = (this._manager.branch.getBoolPref("dayt.panel.enabled") && today.hasChildNodes() && !today.hasAttribute("collapsed"));
    showing["extended"] = (this._manager.branch.getBoolPref("dayf.panel.enabled") && extended.hasChildNodes() && !extended.hasAttribute("collapsed"));
    
    //show-hide spacer
    var cShowing = (showing["deck"] || showing["current"] || showing["radar"] || showing["swa"]);
    var fShowing = (showing["deck"] || showing["today"] || showing["extended"]);
    if (cShowing && fShowing)
      spacer.removeAttribute("collapsed");
    else
      spacer.setAttribute("collapsed", "true");
        
    //remove old first-last attribute
    var box = document.getElementById("ff-box");
    var panels = box.getElementsByTagName("ffpanel");
    for (var x=0; x<panels.length; x++) {
      panels[x].removeAttribute("ff-first");
      panels[x].removeAttribute("ff-last");
    }
    
    if (!showing["deck"]) {
      //set first attribute
      if (showing["swa"])
        swa.setAttribute("ff-first", "true");    
      else if (showing["radar"])
        radar.setAttribute("ff-first", "true");
      else if (showing["current"])
        current.setAttribute("ff-first", "true");
      else if (showing["today"])
        today.firstChild.setAttribute("ff-first", "true");
      else if (showing["extended"])
        extended.firstChild.setAttribute("ff-first", "true");

      //set last attribute
      if (showing["extended"])
        extended.lastChild.setAttribute("ff-last", "true");
      else if (showing["today"])
        today.lastChild.setAttribute("ff-last", "true");
      else if (showing["current"])
        current.setAttribute("ff-last", "true");
      else if (showing["radar"])
        radar.setAttribute("ff-last", "true");
      else if (showing["swa"])
        swa.setAttribute("ff-last", "true");
    }
  },
  
  _getFileURL: function Forecastfox__getFileURL(aName)
  {
    //get file
    var file = this._manager.disk.get(aName, ffIDisk.TYPE_CACHE);
    if (!file)
      return "";
         
    //convert to a URI and return spec
    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    return ios.newFileURI(file).spec;  
  }
}
