/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
const nsIFilePicker = Components.interfaces.nsIFilePicker;
var gOptions = null;

function optionsLoad()
{
  gOptions = new OptionsModule();
  gOptions.start();
}

function optionsUnload()
{
  gOptions.stop();
  gOptions = null;
}

function OptionsModule()
{
  this.wrappedJSObject = this;
  return this;
}

//XXX simplify code
OptionsModule.prototype = {
  _manager: null,
  mAlert: null,
  mPositionMax: null,
  mErrors: null,
  
  start: function OptionsModule_start()
  {
    try {
      this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
      this.mErrors = new Array();
          
      //stop rotating profiles
      this._manager.profiles.rotate(this._manager.profiles.ROTATE_STOP);
  
      //add observers      
      var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obs.addObserver(this, "forecastfox:manager", false);
      
      // Set up the display for the prefs
      this.populateProfiles();
      this.populateOptions();
      this.populateIcons(true);
      
      //enable-disable slider options
      if (!this._manager.hasAlertService) {
        document.getElementById("cc-slider").setAttribute("hidden", true);
        document.getElementById("swa-slider").setAttribute("hidden", true);
      };
      
      // set the validate labels
      this.populateValidLabels();

      //select general page
      var pages = document.getElementById("ff-list-pages");
      pages.selectedItem = pages.childNodes[0];
    } catch (e) {
      this._manager.disk.recordError("Message: An error has occured while initializing the options dialog.", e);
      var strings = document.getElementById("ff-bundle-options");
      alert(strings.getString("ff.error.options")+"\n"+strings.getString("ff.error.contact"));
    }
  },
  
  stop: function OptionsModule_stop()
  {
    //start rotating profiles
    this._manager.profiles.rotate(this._manager.profiles.ROTATE_START);
 
    //remove observers
    try {
      var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obs.removeObserver(this, "forecastfox:manager");
      obs = null;
    } catch(e) {};
    
    //release references
    this._manager = null;
    this.mAlert = null;
    this.mPositionMax = null;
    this.mErrors = null;
  },
  
  units: function OptionsModule_units()
  {
    var selected = document.getElementById("ff-group-units").selectedItem.value;
    window.openDialog("chrome://forecastfox/content/units.xul", "units", "chrome, modal", [selected]);     
  },
  
  checkLocid: function OptionsModule_checkLocid()
  {
    if (this._manager.branch.getCharPref("general.locid") == "00000") {
      // switch to the first page
      var list = document.getElementById("ff-list-pages");
      list.selectedItem = list.childNodes[0];
      this.switchPage(list);
      
      // then highlight the location id
      var locid = document.getElementById("ff-text-code");
      locid.focus();
      locid.select();
    };
  },
  
  switchPage: function OptionsModule_switchPage(aList)
  {
    var deck = document.getElementById("ff-deck-pages");
    var page = aList.selectedIndex;
    deck.selectedIndex = page;  
  },
  
  populateProfiles: function OptionsModule_populateProfiles()
  {
    var list = document.getElementById("ff-list-profs");
    var popup = document.getElementById("ff-popup-profs");    
    var current = this._manager.getUTFPref("profile.current");
    var count = this._manager.profiles.getProfileNum();
    var names = new Array();
    for (var x = 0; x < count; x++)
    	names[x] = this._manager.profiles.getProfile(x);
    names.sort();
    var item;

    //remove old profiles
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    //append new profiles     
    for (x = 0; x < count; x++) {
      item = document.createElement("menuitem");
      item.setAttribute("id", names[x]);
      item.setAttribute("label", names[x]);
      item.setAttribute("type", "prof");
    
      if (names[x] == current)
        list.selectedItem = item;
      
      popup.appendChild(item);
    }
    
    // if the current profile isn't around, select the first
    // one in the list, and switch profiles to it.
    if (list.selectedItem == null) {
      list.selectedItem = popup.firstChild;
      this._manager.setUTFPref("profile.current", list.selectedItem.getAttribute("id"));
    }
  },
  
  populateIcons: function OptionsModule_populateIcons(aRebuild)
  {
    var list = document.getElementById("ff-list-icons");
    var current = this._manager.icons.current.id;
    var icons = this._manager.icons.getItemList({});

    //save current
    var save = null;
    if (list.selectedIndex >= 0)
      save = list.selectedItem.value;
    else
      save = current;
      
    //clear list
    while (list.hasChildNodes() && aRebuild)
      list.removeChild(list.lastChild); 
         
    //populate list    
    for (var x = 0; x < icons.length; x++) {
      var item = aRebuild ? list.appendItem(icons[x].name, icons[x].id) : list.getItemAtIndex(x);
    
      //set selection
      if (icons[x].id == save) {
         //XXX work around for bug 250123 (https://bugzilla.mozilla.org/show_bug.cgi?id=250123)
         list.ensureElementIsVisible(item);
         list.selectedItem = item;
      }
              
      //set current
      if (icons[x].id == current)
        item.setAttribute("icon", "true");
      else
        item.removeAttribute("icon");     
    };   

    //set selected if not already
    item = list.childNodes[0];    
    if (list.selectedIndex < 0) {
      list.selectedItem = item;
    };
    
    //do select function
    this.iconSelect();
  },

  
  populateOptions: function OptionsModule_populateOptions()
  {
    //general
    this.setElement("ff-text-code", "general.locid", "Char", null);
    this.setElement("ff-group-units", "units.current", "Complex", "units");
    this.populateBars();
    this.populatePositions();
  
    //swa panel
    this.setElement("ff-chk-swa-panel", "swa.panel.enabled", "Bool", null);
    this.setElement("ff-list-swa-panel-display", "swa.panel.display", "Int", "ff-popup-swa-panel-display");
  
    //swa tooltip 
    this.setElement("ff-chk-swa-tooltip", "swa.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-swa-tooltip-display", "swa.tooltip.display", "Int", "ff-popup-swa-tooltip-display");   
  
    //swa slider
    this.setElement("ff-chk-swa-slider", "swa.slider.enabled", "Bool", null);
    this.setElement("ff-list-swa-slider-display", "swa.slider.display", "Int", "ff-popup-swa-slider-display");
    this.setElement("ff-text-swa-slider", "swa.slider.freq", "Int", null);

    //radar panel
    this.setElement("ff-chk-radar-panel", "radar.panel.enabled", "Bool", null);
    this.setElement("ff-list-radar-panel-display", "radar.panel.display", "Int", "ff-popup-radar-panel-display");
  
    //radar tooltip 
    this.setElement("ff-chk-radar-tooltip", "radar.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-radar-tooltip-display", "radar.tooltip.display", "Int", "ff-popup-radar-tooltip-display");   
        
    //current panel
    this.setElement("ff-chk-cc-panel", "cc.panel.enabled", "Bool", null);
    this.setElement("ff-list-cc-panel-display", "cc.panel.display", "Int", "ff-popup-cc-panel-display");
  
    //current tooltip 
    this.setElement("ff-chk-cc-tooltip", "cc.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-cc-tooltip-display", "cc.tooltip.display", "Int", "ff-popup-cc-tooltip-display");   
  
    //current slider
    this.setElement("ff-chk-cc-slider", "cc.slider.enabled", "Bool", null);
    this.setElement("ff-list-cc-slider-display", "cc.slider.display", "Int", "ff-popup-cc-slider-display");
    this.setElement("ff-text-cc-slider", "cc.slider.freq", "Int", null);
  
    //today's panel
    this.setElement("ff-chk-dayt-panel", "dayt.panel.enabled", "Bool", null);  
    this.setElement("ff-list-dayt-panel-display", "dayt.panel.display", "Int", "ff-popup-dayt-panel-display");
    this.setElement("ff-list-dayt-panel-mode", "dayt.panel.mode", "Int", "ff-popup-dayt-panel-mode");
   
    //today's tooltip
    this.setElement("ff-chk-dayt-tooltip", "dayt.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-dayt-tooltip-display", "dayt.tooltip.display", "Int", "ff-popup-dayt-tooltip-display");       
  
    //today's switch 
    this.setElement("ff-list-dayt-switch", "dayt.panel.switch", "Int", "ff-popup-dayt-switch");
    
    //extended panel
    this.setElement("ff-text-dayf-days", "dayf.panel.days", "Int", null); 
    this.setElement("ff-chk-dayf-panel", "dayf.panel.enabled", "Bool", null); 
    this.setElement("ff-list-dayf-panel-display", "dayf.panel.display", "Int", "ff-popup-dayf-panel-display"); 
    this.setElement("ff-list-dayf-panel-mode", "dayf.panel.mode", "Int", "ff-popup-dayf-panel-mode");             
  
    //extended tooltip
    this.setElement("ff-chk-dayf-tooltip", "dayf.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-dayf-tooltip-display", "dayf.tooltip.display", "Int", "ff-popup-dayt-tooltip-display");       

    //profiles
    this.setElement("ff-chk-profs-rotate", "profile.switch.enabled", "Bool", null);
    this.setElement("ff-text-profs-rotate", "profile.switch.delay", "Int", null);
    
    //links
    this.setElement("ff-list-links-panel", "links.panel", "Char", "ff-popup-links-panel");
    this.setElement("ff-list-links-alert", "links.alert", "Char", "ff-popup-links-alert");
    this.setElement("ff-list-links-context", "links.context", "Char", "ff-popup-links-context");
    this.setElement("ff-list-links-dialog", "links.dialog", "Char", "ff-popup-links-dialog");
    
    this.checkLocid();
  },

  populateBars: function OptionsModule_populateBars()
  {
    // Creates the menuitems for the toolbar selector.
    var win = this._manager.getWindow();
    var toolbars = win.document.getElementsByTagName("toolbar");
    var statusbars = win.document.getElementsByTagName("statusbar");
    var menubars = win.document.getElementsByTagName("menubar");
    var popup = document.getElementById("ff-popup-bars");
    var x, bar, item, val, list;
    
    // first remove the toolbars already there...
    while (popup.hasChildNodes())
      popup.removeChild(popup.firstChild);
          
    toolbars = concat(toolbars, statusbars);
    toolbars = concat(toolbars, menubars);
    for (x=0; x<toolbars.length; x++) {
      bar = toolbars[x];
      
      //do not include find toolbar
      if (bar.getAttribute("id") == "FindToolbar")
        continue;
             
      item = document.createElement("menuitem");
      item.setAttribute("id", bar.getAttribute("id"));
      item.value = bar.getAttribute("id");         
      if (bar.hasAttribute("toolbarname"))
        item.setAttribute("label", bar.getAttribute("toolbarname"));
      else
        item.setAttribute("label", bar.getAttribute("id"));                 
      popup.appendChild(item);
    }
    
    this.setElement("ff-list-bars", "general.bar", "Char", "ff-popup-bars");    
  },

  populatePositions: function OptionsModule_populatePositions()
  {
    if (!document.getElementById("ff-list-bars").selectedItem) {
      this._manager.branch.setCharPref("general.bar", "status-bar");
      this._manager.branch.setIntPref("general.position", -1);
      this.setElement("ff-list-bars", "general.bar", "Char", "ff-popup-bars");
    };
    
    var barid = document.getElementById("ff-list-bars").selectedItem.getAttribute("id");
    var win = this._manager.getWindow();
    var bar = win.document.getElementById(barid);

    if (!bar)
      this.mPositionMax = 0;
    else {
      var len = bar.childNodes.length;
    
      //don't include ourself in count
      if (win.document.getElementById("ff-box") && win.document.getElementById("ff-box").parentNode == bar)
        len--;
    
      this.mPositionMax = len;
        
    }

    var val = this._manager.branch.getIntPref("general.position");
    var val2 = this._manager.branch.getCharPref("general.bar");
    var text = document.getElementById("ff-text-position");
    var radio = document.getElementsByAttribute("group", "position");
    var group = document.getElementById("ff-group-position");
    var x, el;
    
    //set max value
    text.value = this.mPositionMax;
    
    //mark always last
    if (val == -1) {
      for (x=0; x < radio.length; x++) {
        if (parseInt(radio[x].value) == val) {
          group.selectedItem = radio[x];
          break;
        }
      }
    }
    
    //mark specific position
    else {
      for (x=0; x < radio.length; x++) {
        if (parseInt(radio[x].value) != -1) {
          group.selectedItem = radio[x];
          break;
        }
      }    
      if (barid != val2) {
        text.value = this.mPositionMax;
      } else {
        if (val < 0)
          text.value = this.mPositionMax;
        else if (val > this.mPositionMax)
          text.value = 0;
        else
          text.value = val; 
      }
    }
    
    // update the error label
    el = document.getElementById("ff-text-position-valid");
    el.setAttribute("value", "0-"+this.mPositionMax);
    
    //revalidate position
    setTimeout("gOptions.validate()", 10);
  },
  
  populateValidLabels: function OptionsModule_populateValidLabels() 
  {  
    var el;
    
    el = document.getElementById("ff-text-position-valid");
    el.setAttribute("value", "0-"+this.mPositionMax);
    
    el = document.getElementById("ff-text-swa-slider-valid");
    el.setAttribute("value", "0-16");
    
    el = document.getElementById("ff-text-cc-slider-valid");
    el.setAttribute("value", "0-16");
    
    el = document.getElementById("ff-text-dayf-days-valid");
    el.setAttribute("value", "0-8");
    
    el = document.getElementById("ff-text-profs-rotate-valid");
    el.setAttribute("value", "0-480");
  },
  
  setElement: function OptionsModule_setElement(aName, aPref, aType, aGroup)
  {
    var x, el, els, val;
    
    el = document.getElementById(aName);
    if (aType == "Complex")
      val = this._manager.branch.getComplexValue(aPref,Components.interfaces.nsIPrefLocalizedString).data;
    else
      val = eval("this._manager.branch.get" + aType + "Pref('" + aPref + "')");
    switch (el.localName) {
      case "checkbox":
        el.checked = val;
        break;
      case "textbox":
        el.value = val;
        break;
      case "menulist":
        els = document.getElementById(aGroup).childNodes;        
        for (x=0; x < els.length; x++) {
          if (((aType == "Int") ? parseInt(els[x].value) : els[x].value) == val) {
            el.selectedItem = els[x];
            break;
          }
        }      
        break;            
      case "radiogroup":
        els = document.getElementsByAttribute("group", aGroup);        
        for (x=0; x < els.length; x++) {
          if (((aType == "Int") ? parseInt(els[x].value) : els[x].value) == val) {
            el.selectedItem = els[x];
            break;
          }
        }      
        break;
    }  
  },
  
  setOption: function OptionsModule_setOption(aName, aPref, aType)
  {
    var el = document.getElementById(aName);
    var val;
    switch (el.localName) {
      case "checkbox":
        val = el.checked;
        break;
      case "textbox":
        val = (aType == "Int") ? parseInt(el.value) : this._encodeValue(el.value);
        break;
      case "menulist":
      case "radiogroup":
        val = (aType == "Int") ? parseInt(el.selectedItem.value) : this._encodeValue(el.selectedItem.value);
        break;
    };
    
    switch (aType) {
      case "Complex":
        var support = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);  
        support.data = val;
        this._manager.branch.setComplexValue(aPref,Components.interfaces.nsISupportsString, support);  
        break;
      case "Bool":
      case "Int":
        eval("this._manager.branch.set" + aType + "Pref('" + aPref + "'," + val + ")");
        break;
      case "Char":
      default:
        eval("this._manager.branch.set" + aType + "Pref('" + aPref + "'," + ((aType == "Int") ? val : '"' + val + '"') + ")");
        break;   
    };
  },
  
  selectProfile: function OptionsModule_selectProfile(aList)
  {
    var item = aList.selectedItem.getAttribute("id");
    var popup = document.getElementById("ff-popup-profs");
    popup.hidePopup();
  
    this._manager.setUTFPref("profile.current", item);
  },
  
  validate: function OptionsModule_validate()
  {
    //remove error indicators
    for (var i=0; i < this.mErrors.length; i++)
      this.mErrors[i].removeAttribute("error");
    this.mErrors = new Array();
    
    //check location code
    var el = document.getElementById("ff-text-code");
    if ((el.value == "") || (el.value == "00000"))
      return this.markError(el, 0);
    
    //check position
    el = document.getElementById("ff-text-position");
    var el2 = document.getElementById("ff-text-position-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > this.mPositionMax)
      return this.markError(el, 0, el2);
      
    //check swa slider
    el = document.getElementById("ff-text-swa-slider");
    el2 = document.getElementById("ff-text-swa-slider-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 16)
      return this.markError(el, 1, el2);
    
    //check cc slider
    el = document.getElementById("ff-text-cc-slider");
    el2 = document.getElementById("ff-text-cc-slider-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 16)
      return this.markError(el, 3, el2);
    
    //check extended days
    el = document.getElementById("ff-text-dayf-days");
    el2 = document.getElementById("ff-text-dayf-days-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 8)
      return this.markError(el, 5, el2);
      
    //check profiles rotat
    el = document.getElementById("ff-text-profs-rotate");
    el2 = document.getElementById("ff-text-profs-rotate-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 480)
      return this.markError(el, 6, el2);      

    //return valid      
    return true;
  },

  markError: function OptionsModule_markError(aEl, aIndex, aE2)
  {
    // aE2 is optional: xul object containing a description that will be shown
    //get page list  
    var list = document.getElementById("ff-list-pages");
    
    //mark element
    aEl.setAttribute("error", "true");
    if (aE2)
      aE2.setAttribute("error", "valid");
      // set to valid so css rules don't interfere
    
    //switch to page
    list.selectedItem = list.childNodes[aIndex];     
    this.switchPage(list);
    
    //add to error array
    this.mErrors.push(aEl);
    if (aE2)
      this.mErrors.push(aE2);
    
    return false;
  },
  
  accept: function OptionsModule_accept(aClose)
  {
    if (!this.validate())
      return;

    this.updateOptions();
    
    //make sure we save profile
    this._manager.profiles.save(this._manager.getUTFPref("profile.current"));
    
    if (aClose)  
      window.close();
    else
      document.getElementById("apply").setAttribute("disabled", "true");
  },

  updateOptions: function OptionsModule_updateOptions()
  {
    //general
    this.setOption("ff-text-code", "general.locid", "Char");
    this.setOption("ff-group-units", "units.current", "Complex");
    this.setOption("ff-list-bars", "general.bar", "Char");
    var position = document.getElementById("ff-group-position");
    if (parseInt(position.selectedItem.value) == -1)
      this._manager.branch.setIntPref("general.position", -1);
    else
      this.setOption("ff-text-position", "general.position", "Int");
  
    //swa panel
    this.setOption("ff-chk-swa-panel", "swa.panel.enabled", "Bool");
    this.setOption("ff-list-swa-panel-display", "swa.panel.display", "Int");
    
    //swa tooltip
    this.setOption("ff-chk-swa-tooltip", "swa.tooltip.enabled", "Bool");
    this.setOption("ff-list-swa-tooltip-display", "swa.tooltip.display", "Int");
    
    //swa slider
    this.setOption("ff-chk-swa-slider", "swa.slider.enabled", "Bool");
    this.setOption("ff-list-swa-slider-display", "swa.slider.display", "Int");
    this.setOption("ff-text-swa-slider", "swa.slider.freq", "Int");
  
    //radar panel
    this.setOption("ff-chk-radar-panel", "radar.panel.enabled", "Bool");
    this.setOption("ff-list-radar-panel-display", "radar.panel.display", "Int");
    
    //radar tooltip
    this.setOption("ff-chk-radar-tooltip", "radar.tooltip.enabled", "Bool");
    this.setOption("ff-list-radar-tooltip-display", "radar.tooltip.display", "Int");
          
    //current conditions panel
    this.setOption("ff-chk-cc-panel", "cc.panel.enabled", "Bool");
    this.setOption("ff-list-cc-panel-display", "cc.panel.display", "Int");
    
    //current conditions tooltip
    this.setOption("ff-chk-cc-tooltip", "cc.tooltip.enabled", "Bool");
    this.setOption("ff-list-cc-tooltip-display", "cc.tooltip.display", "Int");
    
    //current conditions slider
    this.setOption("ff-chk-cc-slider", "cc.slider.enabled", "Bool");
    this.setOption("ff-list-cc-slider-display", "cc.slider.display", "Int");
    this.setOption("ff-text-cc-slider", "cc.slider.freq", "Int");
      
    //today's panel
    this.setOption("ff-chk-dayt-panel", "dayt.panel.enabled", "Bool");
    this.setOption("ff-list-dayt-panel-display", "dayt.panel.display", "Int");
    this.setOption("ff-list-dayt-panel-mode", "dayt.panel.mode", "Int");
      
    //today's tooltip
    this.setOption("ff-chk-dayt-tooltip", "dayt.tooltip.enabled", "Bool");
    this.setOption("ff-list-dayt-tooltip-display", "dayt.tooltip.display", "Int");
  
    //today's switch
    this.setOption("ff-list-dayt-switch", "dayt.panel.switch", "Int");
    
    //extended panel
    this.setOption("ff-text-dayf-days", "dayf.panel.days", "Int");
    this.setOption("ff-chk-dayf-panel", "dayf.panel.enabled", "Bool");
    this.setOption("ff-list-dayf-panel-display", "dayf.panel.display", "Int");
    this.setOption("ff-list-dayf-panel-mode", "dayf.panel.mode", "Int");
      
    //extended tooltip
    this.setOption("ff-chk-dayf-tooltip", "dayf.tooltip.enabled", "Bool");
    this.setOption("ff-list-dayf-tooltip-display", "dayf.tooltip.display", "Int");
    
    //profiles
    this.setOption("ff-chk-profs-rotate", "profile.switch.enabled", "Bool");
    this.setOption("ff-text-profs-rotate", "profile.switch.delay", "Int");
    
    //links
    this.setOption("ff-list-links-panel", "links.panel", "Char");
    this.setOption("ff-list-links-alert", "links.alert", "Char");
    this.setOption("ff-list-links-context", "links.context", "Char");
    this.setOption("ff-list-links-dialog", "links.dialog", "Char");
  },
  
  _encodeValue: function OptionsModule__encodeValue(aValue)
  {
    // only encode the strings
    if (typeof aValue != "string")
      return aValue;
    
    // prepare the quotes so that we can use eval
    return trim(aValue.replace(/"/g,"\\\"").replace(/'/g,"\\\'"));
  },
  
  importSettings: function OptionsModule_importSettings()
  {
    this.mProfileLoading = true;
    this._manager.profiles.open(this._manager.profiles.OPEN_IMPORT, window);
    this.mProfileLoading = false;
    this.populateProfiles();
    this.populateOptions();
  },
  
  exportSettings: function OptionsModule_exportSettings()
  {
    this._manager.profiles.open(this._manager.profiles.OPEN_EXPORT, window);
  },
  
  iconSelect: function OptionsModule_iconSelect()
  {
    var uninstall = document.getElementById("ff-btn-icons-uninstall");
    var select = document.getElementById("ff-btn-icons-select");
    var preview = document.getElementById("ff-preview-image");
    var author = document.getElementById("ff-link-icon-author");
    var version = document.getElementById("ff-lbl-icon-version");
    
    //remove values    
    uninstall.setAttribute("disabled", "true");
    select.setAttribute("disabled", "true");
    preview.src = "chrome://forecastfox/skin/images/preview.png";
    version.value = "";
    author.label = ""; 
    author.removeAttribute("tooltiptext");
    author.href = "";
    
    var list = document.getElementById("ff-list-icons");          
    if (list.selectedIndex < 0)
      return;
      
    //fill out metadata
    var pack = this._manager.icons.getItemForId(list.selectedItem.value);
    if (pack.getPreview() != "")
      preview.src = pack.getPreview();
    version.value = pack.version;
    author.label = pack.author;
    author.setAttribute("tooltiptext", pack.website);
    author.href = pack.website;
    
    //enable uninstall
    var current = this._manager.icons.current.id;    
    if (pack.id != "default")
      uninstall.removeAttribute("disabled");
      
    //enable select
    if (pack.id != current)
      select.removeAttribute("disabled");
  },
  
  iconInstall: function OptionsModule_iconInstall()
  {
    var alerts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var bundle = document.getElementById("ff-bundle-options"); 
           
    //get file
    var picker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);  
    picker.appendFilter(bundle.getString("ff.options.icons.filter"), "*.jar");
    picker.defaultExtension = ".jar";
    picker.init(window, bundle.getString("ff.options.icons.picker"), nsIFilePicker.modeOpen);
    
    // get the file and its contents
    var res = picker.show();
    if (res == nsIFilePicker.returnCancel)
      return;

    //perform install
    var rv = this._manager.icons.installItem(picker.file);
    if (!rv) {
      alerts.alert(window, bundle.getString("ff.options.icons.picker"), bundle.getString("ff.icons.failed"));
    } else {
      alerts.alert(window, bundle.getString("ff.options.icons.picker"), bundle.getString("ff.icons.success"));
    };
  },
  
  iconUninstall: function OptionsModule_iconUninstall()
  {
    var alerts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var bundle = document.getElementById("ff-bundle-options"); 
    var list = document.getElementById("ff-list-icons");              
    var name = list.selectedItem.label;
    var current = this._manager.icons.current.id;    
    var id = list.selectedItem.value;    
    var rv = alerts.confirm(window, bundle.getString("ff.options.icons.picker"), bundle.getFormattedString("ff.icons.uninstall", [name]));
    if (!rv)
      return;
    
    //select default pack
    if (id == current)
      this._manager.setUTFPref("icons.current", this._manager.icons.getDefaultItem().id);
    
    //perform uninstall
    try {
      var rv = this._manager.icons.uninstallItem(id);
    } catch(e) {
      this._manager.disk.recordError("ERROR: icon pack uninstall error.", e);
    };
  },
  
  iconSet: function OptionsModule_iconSet()
  {   
    //remove attribute on list items
    var list = document.getElementById("ff-list-icons");              
    for (var x=0; x<list.childNodes.length; x++)
      list.childNodes[x].removeAttribute("icon");
        
    //set icon
    this._manager.setUTFPref("icons.current", list.selectedItem.value);   
  },
  
  observe: function OptionsModule_observe(aSubject, aTopic, aData)
  {
    var method = null;
    var data = null;
    if (aData.indexOf(",") != -1) {
      method = aData.substring(0, aData.indexOf(","));
      data = aData.substring(aData.indexOf(",")+1, aData.length);
    }
    else
      method = aData.substring(0, aData.length);
      
    switch(method) {
      case "profile":
        this.populateProfiles();
        this.populateOptions();
        this.populateIcons(false);
        break;
      case "icons":
        this.populateIcons(data != "changed");
        break;
    };
  }
}
  
function updateButtons(aEvent)
{
  // don't enable the apply button for button presses
  if (aEvent) { 
    if ((aEvent.originalTarget.localName == "button") ||
        (aEvent.originalTarget.localName == "fflink") ||
        (aEvent.originalTarget.getAttribute("type") == "prof"))
    return;
  }
   
  document.getElementById("apply").removeAttribute("disabled");
}

function concat(c1, c2)
{
  // Concats too collections into an array.
  var c3 = new Array(c1.length + c2.length);
  var x,y = 0;

  for (x = 0; x < c1.length; x++)
    c3[y++] = c1[x];

  for (x = 0; x < c2.length; x++)
    c3[y++] = c2[x];

  return c3;
}

// trims trailing and leading white space
function trim(aString) { 
  aString = aString.replace(/^\s+/g, "");
  return aString.replace(/\s+$/g, "");
}
