/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gUnits = null;

function unitsLoad()
{
  gUnits = new UnitsModule();
  gUnits.start();
};

function unitsUnload()
{
  gUnits.stop();
  gUnits = null;
};

function UnitsModule() {};
UnitsModule.prototype = {
  _manager: null,
  _type: null,
  _bundle: null,
  _rows: null,
  _defaults: null,
    
  start: function UnitsModule_start()
  {
    //setup objects
    this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
    this._type = window.arguments[0];
    this._bundle = document.getElementById("ff-bundle-units"); 
    
    //get default branch 
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    this._defaults = pbs.getDefaultBranch("forecastfox.");
             
    //set description
    document.getElementById("ff-units-header").title = this._bundle.getString("ff.units." + this._type);

    //setup display
    this._rows = document.getElementsByTagName("row");
    for (var x=0; x<this._rows.length; x++) {
		  this._populateRow(this._rows[x]);
    };
    
  },
  
  stop: function UnitsModule_stop()
  {
    this._manager = null;  
    this._type = null;
    this._bundle = null;
    this._rows = null;
    this._defaults = null;
  },

  update: function UnitsModule_update(aEvent)
  {
    //do nothing on button press
    if (aEvent) { 
      if (aEvent.originalTarget.localName == "button")
        return;
    };
    
    //find the row that was updated
    var node = aEvent.originalTarget;
    while (node.localName != "row" && node.localName != "window")
      node = node.parentNode;
      
    //do update for row
    if (node.localName == "row") {
      var mask = node.getAttribute("id");
      var selected = node.getElementsByTagName("menulist")[0].selectedItem.value;
      var abbr = document.getElementById("ff-units-" + mask + "-abbr");
      abbr.setAttribute("value", this._manager.parser.getConverter(mask, this._type, selected, null));        
    };
    
  },
  
  accept: function UnitsModule_accept()
  {
    var support = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
        
    for (var x=0; x<this._rows.length; x++) {
      var mask = this._rows[x].getAttribute("id"); 
      var current = this._manager.branch.getComplexValue("units." + mask + "." + this._type, Components.interfaces.nsIPrefLocalizedString).data;      
      var list = this._rows[x].getElementsByTagName("menulist")[0];  
      var selected = list.selectedItem.getAttribute("value");
      
      if (selected != current) {
        support.data = selected;
        this._manager.branch.setComplexValue("units." + mask + "." + this._type, Components.interfaces.nsISupportsString, support);
      };    
    };
    
    window.close();  
  },
  
  _populateRow: function UnitsModule__populateRow(aRow)
  {
    //get the menulist and popup
    var list = aRow.getElementsByTagName("menulist")[0];
    var popup = aRow.getElementsByTagName("menupopup")[0];
    var mask = aRow.getAttribute("id");    
    var abbr = document.getElementById("ff-units-" + mask + "-abbr");        
   
    //get an array of converters for unit and mask
    var converters = this._manager.parser.getConverters(mask, this._type, {});
    var selected = this._manager.branch.getComplexValue("units." + mask + "." + this._type, Components.interfaces.nsIPrefLocalizedString).data;   
    var defaults = this._defaults.getComplexValue("units." + mask + "." + this._type, Components.interfaces.nsIPrefLocalizedString).data;
    for (var x=0; x<converters.length; x++) { 
      
      //create item
      var item = document.createElement("menuitem");
      try {
        var label = this._bundle.getString("ff.units." + mask + "." + converters[x]);
      } catch(e) {
        label = converters[x];
      };
      item.setAttribute("label", label);
      item.setAttribute("value", converters[x]);
        
      //mark the default converter
      if (item.getAttribute("value") == defaults)
        item.setAttribute("default", "true");
        
      //append converter to list        
      popup.appendChild(item);
      
      //select the users converter
      if (item.getAttribute("value") == selected) {
        list.selectedItem = item;
        abbr.setAttribute("value", this._manager.parser.getConverter(mask, this._type, converters[x], null));
      }
    };
  }
}